@@ -402,69 +402,6 @@ void AAIMap::ReadMapCacheFile()
 	}
 
 
-	/*char filename[500];
-
-	STRCPY(buffer, MAIN_PATH);
-	STRCAT(buffer, MAP_CFG_PATH);
-	STRCAT(buffer, cb->GetMapName());
-	ReplaceExtension(buffer, filename, sizeof(filename), ".cfg");
-
-	ai->cb->GetValue(AIVAL_LOCATE_FILE_R, filename);
-
-	if((file = fopen(filename, "r")) != NULL)
-	{
-		// read map type
-		fscanf(file, "%s ", buffer);
-
-		if(!strcmp(buffer, "LAND_MAP"))
-			map_type = LAND_MAP;
-		else if(!strcmp(buffer, "LAND_WATER_MAP"))
-			map_type = LAND_WATER_MAP;
-		else if(!strcmp(buffer, "WATER_MAP"))
-			map_type = WATER_MAP;
-		else
-			map_type = UNKNOWN_MAP;
-
-		if(map_type >= 0 && map_type <= WATER_MAP)
-		{
-			this->map_type = (MapType) map_type;
-
-			SNPRINTF(buffer, buffer_sizeMax, "%s detected", GetMapTypeTextString(map_type));
-
-			if(bt->aai_instances == 1)
-				ai->cb->SendTextMsg(buffer, 0);
-		}
-		else
-			loaded = false;
-
-		fclose(file);
-	}
-	else
-		loaded = false;
-
-	if(!loaded)
-	{
-
-		// logging
-		SNPRINTF(buffer, buffer_sizeMax, "%s detected", GetMapTypeTextString(map_type));
-		ai->cb->SendTextMsg(buffer, 0);
-		fprintf(ai->file, "\nAutodetecting map type:\n");
-		fprintf(ai->file, "%s", buffer);
-		fprintf(ai->file, "\n\n");
-
-		// save results to cfg file
-		STRCPY(buffer, MAIN_PATH);
-		STRCAT(buffer, MAP_CFG_PATH);
-		STRCAT(buffer, cb->GetMapName());
-		ReplaceExtension(buffer, map_filename, sizeof(map_filename), ".cfg");
-
-		ai->cb->GetValue(AIVAL_LOCATE_FILE_W, map_filename);
-
-		file = fopen(map_filename, "w+");
-		fprintf(file, "%s\n", GetMapTypeString(this->map_type));
-		fclose(file);
-	}*/
-
 	// determine important unit categories on this map
 	if(cfg->AIR_ONLY_MOD)
 	{
@@ -657,7 +594,7 @@ std::string AAIMap::LocateMapLearnFile(const bool forWriting) const {
 	} else {
 		cb->GetValue(AIVAL_LOCATE_FILE_R, buffer2);
 	}
-	
+
 	return std::string(buffer2);
 }
 
@@ -677,7 +614,7 @@ std::string AAIMap::LocateMapCacheFile(const bool forWriting) const {
 	} else {
 		cb->GetValue(AIVAL_LOCATE_FILE_R, buffer2);
 	}
-	
+
 	return std::string(buffer2);
 }
 
