@@ -42,13 +42,20 @@ else
 fi
 echo "Using $branch as source"
 
-# Each one of these that is set is build when running this script.
-# .tar.bz2 and .tar.gz are built with linux (LF) line endings.
-# .zip and .7z are built with windows (CRLF) line endings.
 dir="spring_${version_string}"
+
+# Each one of these that is set, is built when running this script.
+# Linux archives
+# * linux (LF) line endings
+# * removed files needed for windows installer generation only
+# * GPL compatible
 lzma="spring_${version_string}_src.tar.lzma"
 #tbz="spring_${version_string}_src.tar.bz2"
 tgz="spring_${branch}_src.tar.gz"
+
+# Windows archives
+# * windows (CRLF) line endings (bugged, see TODO below)
+# * contain everything from the GIT repository
 #zip="spring_${version_string}_src.zip"
 #seven_zip="spring_${branch}_src.7z"
 
@@ -67,12 +74,23 @@ include=" \
  $dir/tools/SelectionEditor/ \
  $dir/CMakeLists.txt \
  $dir/tools/unitsync/ \
+ $dir/tools/DemoTool/ \
  $dir/tools/DedicatedServer/"
 
 # On linux, win32 executables are useless.
-# TASClient is windows only.
 exclude_from_all=""
-linux_exclude="${exclude_from_all} $dir/installer/pkzip.exe"
+linux_exclude="${exclude_from_all}
+	${dir}/installer/include/
+	${dir}/installer/sections/
+	${dir}/installer/graphics/
+	${dir}/installer/nsis_plugins/
+	${dir}/installer/*.exe
+	${dir}/installer/*.bat
+	${dir}/installer/*.nsi
+	${dir}/installer/*.nsh
+	${dir}/installer/make_installer.pl
+	${dir}/installer/make_luaui_nsh.py
+	${dir}/installer/springlobby_download.sh"
 linux_include=""
 windows_exclude="${exclude_from_all}"
 windows_include=""
@@ -82,33 +100,38 @@ echo 'Exporting checkout dir with LF line endings'
 git clone -n . lf/$dir
 cd lf/$dir
 git checkout $branch
+git submodule update --init
 cd ..
 [ -n "$linux_exclude" ] && rm -rf $linux_exclude
 [ -n "$lzma" ] && echo "Creating .tar.lzma archive ($lzma)" && \
-	tar --lzma -c -f "../$lzma" $include $linux_include
+	tar --lzma -c -f "../$lzma" $include $linux_include --exclude=.git
 [ -n "$tbz" ] && echo "Creating .tar.bz2 archive ($tbz)" && \
-	tar -c -j -f "../$tbz" $include $linux_include
+	tar -c -j -f "../$tbz" $include $linux_include --exclude=.git
 [ -n "$tgz" ] && echo "Creating .tar.gz archive ($tgz)" && \
-	tar -c -z -f "../$tgz" $include $linux_include
+	tar -c -z -f "../$tgz" $include $linux_include --exclude=.git
 cd ..
 echo 'Cleaning'
 rm -rf lf
 
-### TODO: needs fixing
-# Windows line endings, .zip/.7z package
-#echo 'Exporting checkout dir with CRLF line endings'
-git clone -n . crlf/$dir
-cd crlf/$dir
-git config core.autocrlf true
-git checkout $branch
-cd ..
+if [ -n "$zip" ] || [ -n "$seven_zip" ]; then
+	### TODO: needs fixing (not really using CRLF)
+	# Windows line endings, .zip/.7z package
+	#echo 'Exporting checkout dir with CRLF line endings'
+	git clone -n . crlf/$dir
+	cd crlf/$dir
+	git config core.autocrlf true
+	git checkout $branch
+	git submodule update --init
+	cd ..
+
+	[ -n "$windows_exclude" ] && rm -rf $windows_exclude
+	[ -n "$zip" ] && [ -x /usr/bin/zip ] && echo "Creating .zip archive ($zip)" && \
+		/usr/bin/zip -q -r -u -9 "../$zip" $include $windows_include
+	[ -n "$seven_zip" ] && [ -x /usr/bin/7z ] && echo "Creating .7z archive ($seven_zip)" && \
+		/usr/bin/7z a -t7z -m0=lzma -mx=9 -mfb=64 -md=32m -ms=on "../$seven_zip" $include >/dev/null
+	cd ..
+	echo 'Cleaning'
+	rm -rf crlf
+fi
 
-[ -n "$windows_exclude" ] && rm -rf $windows_exclude
-[ -n "$zip" ] && [ -x /usr/bin/zip ] && echo "Creating .zip archive ($zip)" && \
-	/usr/bin/zip -q -r -u -9 "../$zip" $include $windows_include
-[ -n "$seven_zip" ] && [ -x /usr/bin/7z ] && echo "Creating .7z archive ($seven_zip)" && \
-	/usr/bin/7z a -t7z -m0=lzma -mx=9 -mfb=64 -md=32m -ms=on "../$seven_zip" $include >/dev/null
-cd ..
-echo 'Cleaning'
-rm -rf crlf
 cd ..
