@@ -36,16 +36,7 @@
 #include "Rendering/Textures/Bitmap.h"
 #include "Sim/Misc/LosHandler.h"
 #include "Sim/Misc/RadarHandler.h"
-#include "Sim/Projectiles/PieceProjectile.h"
-#include "Sim/Projectiles/Projectile.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
-#include "Sim/Projectiles/Unsynced/GeoThermSmokeProjectile.h"
-#include "Sim/Projectiles/Unsynced/GfxProjectile.h"
-#include "Sim/Projectiles/Unsynced/WreckProjectile.h"
-#include "Sim/Projectiles/WeaponProjectiles/BeamLaserProjectile.h"
-#include "Sim/Projectiles/WeaponProjectiles/LargeBeamLaserProjectile.h"
-#include "Sim/Projectiles/WeaponProjectiles/LightningProjectile.h"
-#include "Sim/Projectiles/WeaponProjectiles/WeaponProjectile.h"
 #include "Sim/Units/CommandAI/CommandAI.h"
 #include "Sim/Units/CommandAI/LineDrawer.h"
 #include "Sim/Units/Unit.h"
@@ -271,7 +262,7 @@ void CMiniMap::SetMaximizedGeometry()
 	}
 	else {
 		const float mapRatio = (float)gs->mapx / (float)gs->mapy;
-		const float viewRatio = (float)gu->viewSizeX / (float)gu->viewSizeY;
+		const float viewRatio = gu->aspectRatio;
 		if (mapRatio > viewRatio) {
 			xpos = 0;
 			width = gu->viewSizeX;
@@ -974,12 +965,17 @@ void CMiniMap::DrawCircle(const float3& pos, float radius)
 
 void CMiniMap::DrawSquare(const float3& pos, float xsize, float zsize)
 {
-	glBegin(GL_LINE_LOOP);
-		glVertex3f(pos.x + xsize, 0.0f, pos.z + zsize);
-		glVertex3f(pos.x - xsize, 0.0f, pos.z + zsize);
-		glVertex3f(pos.x - xsize, 0.0f, pos.z - zsize);
-		glVertex3f(pos.x + xsize, 0.0f, pos.z - zsize);
-	glEnd();
+	float verts[] = {
+		pos.x + xsize, 0.0f, pos.z + zsize,
+		pos.x - xsize, 0.0f, pos.z + zsize,
+		pos.x - xsize, 0.0f, pos.z - zsize,
+		pos.x + xsize, 0.0f, pos.z - zsize
+
+	};
+	glEnableClientState(GL_VERTEX_ARRAY);
+	glVertexPointer(3, GL_FLOAT, 0, verts);
+	glDrawArrays(GL_LINE_LOOP, 0, 4);
+	glDisableClientState(GL_VERTEX_ARRAY);
 }
 
 
@@ -1090,14 +1086,16 @@ void CMiniMap::DrawForReal()
 			}
 		}
 		glColor4f(1,1,1,0.5f);
-		glBegin(GL_LINES);
+		CVertexArray* va = GetVertexArray();
+		va->Initialize();
+		va->EnlargeArrays(left.size()*2, 0, VA_SIZE_2D0);
 		for(fli = left.begin(); fli != left.end(); fli++) {
 			if(fli->minz < fli->maxz) {
-				DrawInMap2D(fli->base + (fli->dir * fli->minz), fli->minz);
-				DrawInMap2D(fli->base + (fli->dir * fli->maxz), fli->maxz);
+				va->AddVertex2dQ0(fli->base + (fli->dir * fli->minz), fli->minz);
+				va->AddVertex2dQ0(fli->base + (fli->dir * fli->maxz), fli->maxz);
 			}
 		}
-		glEnd();
+		va->DrawArray2d0(GL_LINES);
 	}
 
 	glRotatef(-90.0f, +1.0f, 0.0f, 0.0f); // real 'world' coordinates
@@ -1123,6 +1121,7 @@ void CMiniMap::DrawForReal()
 		if ((drawCommands > 0) && guihandler->GetQueueKeystate()) {
 			selectedUnits.DrawCommands();
 		}
+		lineDrawer.DrawAll();
 	}
 
 	glDisable(GL_DEPTH_TEST);
@@ -1177,12 +1176,18 @@ void CMiniMap::DrawForReal()
 		glBlendFunc((GLenum)cmdColors.MouseBoxBlendSrc(),
 		            (GLenum)cmdColors.MouseBoxBlendDst());
 		glLineWidth(cmdColors.MouseBoxLineWidth());
-		glBegin(GL_LINE_LOOP);
-			DrawInMap2D(oldPos.x, oldPos.z);
-			DrawInMap2D(newPos.x, oldPos.z);
-			DrawInMap2D(newPos.x, newPos.z);
-			DrawInMap2D(oldPos.x, newPos.z);
-		glEnd();
+
+		float verts[] = {
+			oldPos.x, oldPos.z,
+			newPos.x, oldPos.z,
+			newPos.x, newPos.z,
+			oldPos.x, newPos.z,
+		};
+		glEnableClientState(GL_VERTEX_ARRAY);
+		glVertexPointer(2, GL_FLOAT, 0, verts);
+		glDrawArrays(GL_LINE_LOOP, 0, 4);
+		glDisableClientState(GL_VERTEX_ARRAY);
+
 		glLineWidth(1.0f);
 		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	}
