@@ -208,7 +208,6 @@ void CMouseHandler::MouseMove(int x, int y)
 	}
 }
 
-
 void CMouseHandler::MousePress(int x, int y, int button)
 {
 	if (button > NUM_BUTTONS)
@@ -221,16 +220,6 @@ void CMouseHandler::MousePress(int x, int y, int button)
 	if (!game->gameOver)
 		playerHandler->Player(gu->myPlayerNum)->currentStats.mouseClicks++;
 
-	if (button == 4) {
-		if (guihandler->buildSpacing > 0)
-			guihandler->buildSpacing--;
-		return;
-	}
-	if (button == 5) {
-		guihandler->buildSpacing++;
-		return;
-	}
-
  	buttons[button].chorded = buttons[SDL_BUTTON_LEFT].pressed ||
  	                          buttons[SDL_BUTTON_RIGHT].pressed;
 	buttons[button].pressed = true;
@@ -446,7 +435,7 @@ void CMouseHandler::MouseRelease(int x, int y, int button)
 			if (unit && ((unit->team == gu->myTeam) || gu->spectatingFullSelect)) {
 				if (buttons[button].lastRelease < (gu->gameTime - doubleClickTime)) {
 					CUnit* unitM = uh->units[unit->id];
-					if (keys[SDLK_LCTRL] && selectedUnits.selectedUnits.find(unit) != selectedUnits.selectedUnits.end()) {
+					if (keys[SDLK_LCTRL] && selectedUnits.selectedUnits.find((CUnit*)unit) != selectedUnits.selectedUnits.end()) {
 						selectedUnits.RemoveUnit(unitM);
 					} else {
 						selectedUnits.AddUnit(unitM);
@@ -552,12 +541,19 @@ void CMouseHandler::Draw()
 		            (GLenum)cmdColors.MouseBoxBlendDst());
 
 		glLineWidth(cmdColors.MouseBoxLineWidth());
-		glBegin(GL_LINE_LOOP);
-		glVertexf3(camera->pos+dir1U*30+dir1S*30+camera->forward*30);
-		glVertexf3(camera->pos+dir2U*30+dir1S*30+camera->forward*30);
-		glVertexf3(camera->pos+dir2U*30+dir2S*30+camera->forward*30);
-		glVertexf3(camera->pos+dir1U*30+dir2S*30+camera->forward*30);
-		glEnd();
+
+		float3 verts[] = {
+			camera->pos+dir1U*30+dir1S*30+camera->forward*30,
+			camera->pos+dir2U*30+dir1S*30+camera->forward*30,
+			camera->pos+dir2U*30+dir2S*30+camera->forward*30,
+			camera->pos+dir1U*30+dir2S*30+camera->forward*30,
+		};
+
+		glEnableClientState(GL_VERTEX_ARRAY);
+		glVertexPointer(3, GL_FLOAT, 0, verts);
+		glDrawArrays(GL_LINE_LOOP, 0, 4);
+		glDisableClientState(GL_VERTEX_ARRAY);
+
 		glLineWidth(1.0f);
 
 		glPopAttrib();
