@@ -27,6 +27,7 @@
 #include "Sim/Misc/Team.h"
 #include "Map/Ground.h"
 #include "Map/MapDamage.h"
+#include "Map/MapInfo.h"
 #include "Map/ReadMap.h"
 #include "Rendering/GroundDecalHandler.h"
 #include "Rendering/Env/BaseTreeDrawer.h"
@@ -36,10 +37,12 @@
 #include "Sim/Misc/CollisionVolume.h"
 #include "Sim/Misc/DamageArray.h"
 #include "Sim/Misc/LosHandler.h"
+#include "Sim/Misc/SmoothHeightMesh.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/Misc/QuadField.h"
 #include "Sim/MoveTypes/AirMoveType.h"
 #include "Sim/MoveTypes/TAAirMoveType.h"
+#include "Sim/Path/PathManager.h"
 #include "Sim/Projectiles/ExplosionGenerator.h"
 #include "Sim/Projectiles/Projectile.h"
 #include "Sim/Projectiles/PieceProjectile.h"
@@ -79,6 +82,7 @@ bool LuaSyncedCtrl::inCreateFeature = false;
 bool LuaSyncedCtrl::inDestroyFeature = false;
 bool LuaSyncedCtrl::inGiveOrder = false;
 bool LuaSyncedCtrl::inHeightMap = false;
+bool LuaSyncedCtrl::inSmoothMesh = false;
 
 static int heightMapx1;
 static int heightMapx2;
@@ -86,6 +90,8 @@ static int heightMapz1;
 static int heightMapz2;
 static float heightMapAmountChanged;
 
+static float smoothMeshAmountChanged;
+
 
 /******************************************************************************/
 
@@ -213,6 +219,17 @@ bool LuaSyncedCtrl::PushEntries(lua_State* L)
 	REGISTER_LUA_CFUNC(SetHeightMap);
 	REGISTER_LUA_CFUNC(SetHeightMapFunc);
 
+	REGISTER_LUA_CFUNC(LevelSmoothMesh);
+	REGISTER_LUA_CFUNC(AdjustSmoothMesh);
+	REGISTER_LUA_CFUNC(RevertSmoothMesh);
+
+	REGISTER_LUA_CFUNC(AddSmoothMesh);
+	REGISTER_LUA_CFUNC(SetSmoothMesh);
+	REGISTER_LUA_CFUNC(SetSmoothMeshFunc);
+
+	REGISTER_LUA_CFUNC(SetMapSquareTerrainType);
+	REGISTER_LUA_CFUNC(SetTerrainTypeData);
+
 	REGISTER_LUA_CFUNC(SpawnCEG);
 
 	REGISTER_LUA_CFUNC(EditUnitCmdDesc);
@@ -368,16 +385,15 @@ static inline CFeature* ParseFeature(lua_State* L,
 		luaL_error(L, "Incorrect arguments to %s(featureID)", caller);
 	}
 	const int featureID = lua_toint(L, index);
-	const CFeatureSet& fset = featureHandler->GetActiveFeatures();
-	CFeatureSet::const_iterator it = fset.find(featureID);
+	CFeature* f = featureHandler->GetFeature(featureID);
 
-	if (it == fset.end())
+	if (!f)
 		return NULL;
 
-	if (!CanControlFeature(*it)) {
+	if (!CanControlFeature(f)) {
 		return NULL;
 	}
-	return *it;
+	return f;
 }
 
 
@@ -983,7 +999,7 @@ int LuaSyncedCtrl::SetUnitHealth(lua_State* L)
 				}
 				else if (key == "paralyze") {
 					unit->paralyzeDamage = max(0.0f, value);
-					if (unit->paralyzeDamage > unit->maxHealth) {
+					if (unit->paralyzeDamage > (modInfo.paralyzeOnMaxHealth? unit->maxHealth: unit->health)) {
 						unit->stunned = true;
 					} else if (value < 0.0f) {
 						unit->stunned = false;
@@ -1097,7 +1113,7 @@ int LuaSyncedCtrl::SetUnitWeaponState(lua_State* L)
 		return 0;
 	}
 
-	const int weaponNum = luaL_checkint(L,2);
+	const int weaponNum = luaL_checkint(L, 2);
 	if ((weaponNum < 0) || ((size_t)weaponNum >= unit->weapons.size())) {
 		return 0;
 	}
@@ -2025,6 +2041,7 @@ int LuaSyncedCtrl::CreateFeature(lua_State* L)
 		heading = lua_toint(L, 5);
 	}
 
+	int facing = GetFacingFromHeading(heading);
 	int team = CtrlTeam();
 	if (team < 0) {
 		team = -1; // default to global for AllAccessTeam
@@ -2053,7 +2070,7 @@ int LuaSyncedCtrl::CreateFeature(lua_State* L)
 	// use SetFeatureResurrect() to fill in the missing bits
 	inCreateFeature = true;
 	CFeature* feature = new CFeature();
-	feature->Initialize(pos, featureDef, heading, 0, team, allyTeam, "");
+	feature->Initialize(pos, featureDef, heading, facing, team, allyTeam, "");
 	inCreateFeature = false;
 
 	lua_pushnumber(L, feature->id);
@@ -2812,7 +2829,7 @@ int LuaSyncedCtrl::SetHeightMapFunc(lua_State* L)
 
 	if (error != 0) {
 		logOutput.Print("Spring.SetHeightMapFunc: error(%i) = %s",
-		                error, lua_tostring(L, -1));
+				error, lua_tostring(L, -1));
 		lua_error(L);
 	}
 
@@ -2824,6 +2841,277 @@ int LuaSyncedCtrl::SetHeightMapFunc(lua_State* L)
 	return 1;
 }
 
+/******************************************************************************/
+/* smooth mesh manipulation                                                   */
+/******************************************************************************/
+
+static void ParseSmoothMeshParams(lua_State* L, const char* caller, float& factor,
+			   int& x1, int& z1, int& x2, int& z2)
+{
+	float fx1 = 0.0f;
+	float fz1 = 0.0f;
+	float fx2 = 0.0f;
+	float fz2 = 0.0f;
+
+	const int args = lua_gettop(L); // number of arguments
+	if (args == 3) {
+		fx1 = fx2 = luaL_checkfloat(L, 1);
+		fz1 = fz2 = luaL_checkfloat(L, 2);
+		factor    = luaL_checkfloat(L, 3);
+	}
+	else if (args == 5) {
+		fx1    = luaL_checkfloat(L, 1);
+		fz1    = luaL_checkfloat(L, 2);
+		fx2    = luaL_checkfloat(L, 3);
+		fz2    = luaL_checkfloat(L, 4);
+		factor = luaL_checkfloat(L, 5);
+		if (fx1 > fx2) {
+			swap(fx1, fx2);
+		}
+		if (fz1 > fz2) {
+			swap(fz1, fz2);
+		}
+	}
+	else {
+		luaL_error(L, "Incorrect arguments to %s()", caller);
+	}
+
+	// quantize and clamp
+	x1 = (int)max(0 , min(smoothGround->GetMaxX(), (int)(fx1 / smoothGround->GetResolution())));
+	z1 = (int)max(0 , min(smoothGround->GetMaxY(), (int)(fz1 / smoothGround->GetResolution())));
+	x2 = (int)max(0 , min(smoothGround->GetMaxX(), (int)(fx2 / smoothGround->GetResolution())));
+	z2 = (int)max(0 , min(smoothGround->GetMaxY(), (int)(fz2 / smoothGround->GetResolution())));
+
+	return;
+}
+
+
+int LuaSyncedCtrl::LevelSmoothMesh(lua_State *L)
+{
+	float height;
+	int x1, x2, z1, z2;
+	ParseSmoothMeshParams(L, __FUNCTION__, height, x1, z1, x2, z2);
+
+	for (int z = z1; z <= z2; z++) {
+		for (int x = x1; x <= x2; x++) {
+			const int index = (z * smoothGround->GetMaxX()) + x;
+			smoothGround->SetHeight(index, height);
+		}
+	}
+	return 0;
+}
+
+int LuaSyncedCtrl::AdjustSmoothMesh(lua_State *L)
+{
+	float height;
+	int x1, x2, z1, z2;
+	ParseSmoothMeshParams(L, __FUNCTION__, height, x1, z1, x2, z2);
+
+	for (int z = z1; z <= z2; z++) {
+		for (int x = x1; x <= x2; x++) {
+			const int index = (z * smoothGround->GetMaxX()) + x;
+			smoothGround->AddHeight(index, height);
+		}
+	}
+
+	return 0;
+}
+
+int LuaSyncedCtrl::RevertSmoothMesh(lua_State *L)
+{
+	float origFactor;
+	int x1, x2, z1, z2;
+	ParseSmoothMeshParams(L, __FUNCTION__, origFactor, x1, z1, x2, z2);
+
+	const float* origMap = smoothGround->GetOriginalMeshData();
+	const float* currMap = smoothGround->GetMeshData();
+
+	if (origFactor == 1.0f) {
+		for (int z = z1; z <= z2; z++) {
+			for (int x = x1; x <= x2; x++) {
+				const int idx = (z * smoothGround->GetMaxX()) + x;
+				smoothGround->SetHeight(idx, origMap[idx]);
+			}
+		}
+	}
+	else {
+		const float currFactor = (1.0f - origFactor);
+		for (int z = z1; z <= z2; z++) {
+			for (int x = x1; x <= x2; x++) {
+				const int index = (z * smoothGround->GetMaxX()) + x;
+				const float ofh = origFactor * origMap[index];
+				const float cfh = currFactor * currMap[index];
+				smoothGround->SetHeight(index, ofh + cfh);
+			}
+		}
+	}
+
+	return 0;
+}
+
+
+int LuaSyncedCtrl::AddSmoothMesh(lua_State *L)
+{
+	if (!inSmoothMesh) {
+		luaL_error(L, "AddSmoothMesh() can only be called in SetSmoothMeshFunc()");
+	}
+
+	const float xl = luaL_checkfloat(L, 1);
+	const float zl = luaL_checkfloat(L, 2);
+	const float h  = luaL_checkfloat(L, 3);
+
+	// quantize
+	const int x = (int)(xl / smoothGround->GetResolution());
+	const int z = (int)(zl / smoothGround->GetResolution());
+
+	// discard invalid coordinates
+	if ((x < 0) || (x > smoothGround->GetMaxX()) ||
+	    (z < 0) || (z > smoothGround->GetMaxY())) {
+		return 0;
+	}
+
+	const int index = (z * smoothGround->GetMaxX()) + x;
+	const float oldHeight = smoothGround->GetMeshData()[index];
+	smoothMeshAmountChanged += streflop::fabsf(h);
+
+	smoothGround->AddHeight(index, h);
+	// push the new height
+	lua_pushnumber(L, oldHeight + h);
+	return 1;
+}
+
+int LuaSyncedCtrl::SetSmoothMesh(lua_State *L)
+{
+	if (!inSmoothMesh) {
+		luaL_error(L, "SetSmoothMesh() can only be called in SetSmoothMeshFunc()");
+	}
+
+	const float xl = luaL_checkfloat(L, 1);
+	const float zl = luaL_checkfloat(L, 2);
+	const float h  = luaL_checkfloat(L, 3);
+
+	// quantize
+	const int x = (int)(xl / smoothGround->GetResolution());
+	const int z = (int)(zl / smoothGround->GetResolution());
+
+	// discard invalid coordinates
+	if ((x < 0) || (x > smoothGround->GetMaxX()) ||
+	    (z < 0) || (z > smoothGround->GetMaxY())) {
+		return 0;
+	}
+
+	const int index = (z * (smoothGround->GetMaxX())) + x;
+	const float oldHeight = smoothGround->GetMeshData()[index];
+	float height = oldHeight;
+
+	if (lua_israwnumber(L, 4)) {
+		const float t = lua_tofloat(L, 4);
+		height += (h - oldHeight) * t;
+	} else{
+		height = h;
+	}
+
+	const float heightDiff = (height - oldHeight);
+	smoothMeshAmountChanged += streflop::fabsf(heightDiff);
+
+	smoothGround->SetHeight(index, height);
+	lua_pushnumber(L, heightDiff);
+	return 1;
+}
+
+int LuaSyncedCtrl::SetSmoothMeshFunc(lua_State *L)
+{
+	const int args = lua_gettop(L); // number of arguments
+	if ((args < 1) || !lua_isfunction(L, 1)) {
+		luaL_error(L, "Incorrect arguments to Spring.SetSmoothMeshFunc(func, ...)");
+	}
+
+	if (inSmoothMesh) {
+		luaL_error(L, "SetHeightMapFunc() recursion is not permitted");
+	}
+
+	heightMapAmountChanged = 0.0f;
+
+	inSmoothMesh = true;
+	const int error = lua_pcall(L, (args - 1), 0, 0);
+	inSmoothMesh = false;
+
+	if (error != 0) {
+		logOutput.Print("Spring.SetSmoothMeshFunc: error(%i) = %s",
+				error, lua_tostring(L, -1));
+		lua_error(L);
+	}
+
+	lua_pushnumber(L, smoothMeshAmountChanged);
+	return 1;
+}
+
+/******************************************************************************/
+/******************************************************************************/
+
+int LuaSyncedCtrl::SetMapSquareTerrainType(lua_State* L)
+{
+	const int hx = int(luaL_checkfloat(L, 1) / SQUARE_SIZE);
+	const int hz = int(luaL_checkfloat(L, 2) / SQUARE_SIZE);
+
+	if ((hx < 0) || (hx > gs->mapx) || (hz < 0) || (hz > gs->mapy)) {
+		return 0;
+	}
+
+	const int tx = hx >> 1;
+	const int tz = hz >> 1;
+
+	const int ott = readmap->typemap[tz * gs->hmapx + tx];
+	const int ntt = luaL_checkint(L, 3);
+
+	readmap->typemap[tz * gs->hmapx + tx] = std::max(0, std::min(ntt, (CMapInfo::NUM_TERRAIN_TYPES - 1)));
+	pathManager->TerrainChange(hx, hz,  hx + 1, hz + 1);
+
+	lua_pushnumber(L, ott);
+	return 1;
+}
+
+int LuaSyncedCtrl::SetTerrainTypeData(lua_State* L)
+{
+	const int args = lua_gettop(L);
+	const int tti = luaL_checkint(L, 1);
+
+	if (tti < 0 || tti >= CMapInfo::NUM_TERRAIN_TYPES) {
+		lua_pushboolean(L, false);
+		return 1;
+	}
+
+	CMapInfo::TerrainType* tt = const_cast<CMapInfo::TerrainType*>(&mapInfo->terrainTypes[tti]);
+
+	if (args >= 2) { tt->tankSpeed  = luaL_checkfloat(L, 2); }
+	if (args >= 3) { tt->kbotSpeed  = luaL_checkfloat(L, 3); }
+	if (args >= 4) { tt->hoverSpeed = luaL_checkfloat(L, 4); }
+	if (args >= 5) { tt->shipSpeed  = luaL_checkfloat(L, 5); }
+
+	/*
+	if (!mapDamage->disabled) {
+		CBasicMapDamage* bmd = dynamic_cast<CBasicMapDamage*>(mapDamage);
+
+		if (bmd != NULL) {
+			tt->hardness = luaL_checkfloat(L, 6);
+			bmd->invHardness[tti] = 1.0f / tt->hardness;
+		}
+	}
+	*/
+
+	// update all map-squares set to this terrain-type (slow)
+	for (int tx = 0; tx < gs->hmapx; tx++) {
+		for (int tz = 0; tz < gs->hmapy; tz++) {
+			if (readmap->typemap[tz * gs->hmapx + tx] == tti) {
+				pathManager->TerrainChange((tx << 1), (tz << 1),  (tx << 1) + 1, (tz << 1) + 1);
+			}
+		}
+	}
+
+	lua_pushboolean(L, true);
+	return 1;
+}
+
 /******************************************************************************/
 /******************************************************************************/
 
