@@ -64,12 +64,14 @@
 #include "Sim/Units/Groups/Group.h"
 #include "Sim/Units/Groups/GroupHandler.h"
 #include "LogOutput.h"
+#include "Util.h"
 #include "NetProtocol.h"
 #include "Sound/Sound.h"
 #include "Sound/AudioChannel.h"
 #include "Sound/Music.h"
 
 #include "FileSystem/FileHandler.h"
+#include "FileSystem/FileSystemHandler.h"
 #include "FileSystem/FileSystem.h"
 #include "ConfigHandler.h"
 
@@ -1172,7 +1174,7 @@ int LuaUnsyncedCtrl::SetUnitNoMinimap(lua_State* L)
 
 int LuaUnsyncedCtrl::SetUnitNoSelect(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(sel); // SetUnitNoSelect
+//	GML_RECMUTEX_LOCK(sel); // SetUnitNoSelect - this mutex is already locked (lua)
 
 	if (CLuaHandle::GetActiveHandle()->GetUserMode()) {
 		return 0;
@@ -1685,23 +1687,27 @@ int LuaUnsyncedCtrl::Restart(lua_State* L)
 	const string arguments = luaL_checkstring(L, 1);
 	const string script = luaL_checkstring(L, 2);
 
+	const std::string springFullName = (Platform::GetBinaryFile());
+	// LogObject() << "Args: " << arguments;
 	if (!script.empty())
 	{
-		std::ofstream scriptfile((FileSystemHandler::GetInstance().GetWriteDir()+"/script.txt").c_str());
+		const std::string scriptFullName = FileSystemHandler::GetInstance().GetWriteDir()+"script.txt";
+		// LogObject() << "Writing script to: " << scriptFullName;
+		std::ofstream scriptfile(scriptFullName.c_str());
 		scriptfile << script;
 		scriptfile.close();
 		//FIXME: ugly
 		if (arguments.empty())
-			EXECLP(Platform::GetBinaryFile().c_str(), Platform::GetBinaryFile().c_str(), (FileSystemHandler::GetInstance().GetWriteDir()+"/script.txt").c_str(), NULL);
+			EXECLP(springFullName.c_str(), Quote(springFullName).c_str(), Quote(scriptFullName).c_str(), NULL);
 		else
-			EXECLP(Platform::GetBinaryFile().c_str(), Platform::GetBinaryFile().c_str(), arguments.c_str(), (FileSystemHandler::GetInstance().GetWriteDir()+"/script.txt").c_str(), NULL);
+			EXECLP(springFullName.c_str(), Quote(springFullName).c_str(), arguments.c_str(), Quote(scriptFullName).c_str(), NULL);
 	}
 	else
 	{
 		if (arguments.empty())
-			EXECLP(Platform::GetBinaryFile().c_str(), Platform::GetBinaryFile().c_str(), NULL);
+			EXECLP(springFullName.c_str(), Quote(springFullName).c_str(), NULL);
 		else
-			EXECLP(Platform::GetBinaryFile().c_str(), Platform::GetBinaryFile().c_str(), arguments.c_str(), NULL);
+			EXECLP(springFullName.c_str(), Quote(springFullName).c_str(), arguments.c_str(), NULL);
 	}
 	LogObject() << "Error in Restart: " << strerror(errno);
 	lua_pushboolean(L, false);
@@ -1791,7 +1797,7 @@ int LuaUnsyncedCtrl::SetUnitDefImage(lua_State* L)
 
 int LuaUnsyncedCtrl::SetUnitGroup(lua_State* L)
 {
-	GML_RECMUTEX_LOCK(group); // SetUnitGroup
+//	GML_RECMUTEX_LOCK(group); // SetUnitGroup - this mutex is already locked (lua)
 
 	if (!CheckModUICtrl()) {
 		return 0;
@@ -2242,7 +2248,7 @@ int LuaUnsyncedCtrl::MarkerAddPoint(lua_State* L)
 	                 lua_tofloat(L, 2),
 	                 lua_tofloat(L, 3));
 	const string text = luaL_optstring(L, 4, "");
-	const bool onlyLocal = bool(luaL_optnumber(L, 5, 0));
+	const bool onlyLocal = bool(luaL_optnumber(L, 5, 1));
 
 	if (onlyLocal) {
 		inMapDrawer->LocalPoint(pos, text, gu->myPlayerNum);
