@@ -5,10 +5,26 @@
 
 #include "Util.h"
 #include "LogOutput.h"
+
+
+unsigned char TeamBase::teamDefaultColor[10][4] =
+{
+	{ 90, 90, 255, 255}, //blue
+	{ 200, 0, 0, 255}, //red
+	{ 255, 255, 255, 255}, //white
+	{ 38, 155, 32, 255}, //green
+	{ 7, 31, 125, 255}, //blue
+	{ 150, 10, 180, 255}, //purple
+	{ 255, 255, 0, 255}, //yellow
+	{ 50, 50, 50, 255}, //black
+	{ 152, 200, 220, 255}, //ltblue
+	{ 171, 171, 131, 255} //tan
+};
+
 TeamBase::TeamBase() :
 leader(-1),
 handicap(1),
-startPos(100,100,100),
+startPos(-100,-100,-100),
 teamAllyteam(-1),
 startMetal(-1.0),
 startEnergy(-1.0)
@@ -40,6 +56,16 @@ void TeamBase::SetValue(const std::string& key, const std::string& value)
 		}
 		color[3] = 255;
 	}
+	else if (key == "startposx")
+	{
+		if (!value.empty())
+			startPos.x = atoi(value.c_str());
+	}
+	else if (key == "startposz")
+	{
+		if (!value.empty())
+			startPos.z = atoi(value.c_str());
+	}
 	else
 		customValues[key] = value;
 }
