@@ -1,6 +1,8 @@
 #include "StdAfx.h"
 #include "AAirMoveType.h"
+
 #include "Sim/Units/Unit.h"
+#include "Sim/Units/CommandAI/CommandAI.h"
 
 
 CR_BIND_DERIVED_INTERFACE(AAirMoveType, AMoveType);
@@ -13,6 +15,7 @@ CR_REG_METADATA(AAirMoveType, (
 		CR_MEMBER(wantedHeight),
 
 		CR_MEMBER(collide),
+		CR_MEMBER(useSmoothMesh),
 		CR_MEMBER(lastColWarning),
 		CR_MEMBER(lastColWarningType),
 
@@ -29,6 +32,7 @@ AAirMoveType::AAirMoveType(CUnit* unit) :
 	reservedLandingPos(-1,-1,-1),
 	wantedHeight(80),
 	collide(true),
+	useSmoothMesh(false),
 	lastColWarning(0),
 	lastColWarningType(0),
 	autoLand(true)
@@ -44,6 +48,19 @@ AAirMoveType::~AAirMoveType()
 	}
 }
 
+bool AAirMoveType::UseSmoothMesh() const
+{
+	if (useSmoothMesh)
+	{
+		const bool onTransportMission = !owner->commandAI->commandQue.empty() && ((owner->commandAI->commandQue.front().id == CMD_LOAD_UNITS)  || (owner->commandAI->commandQue.front().id == CMD_UNLOAD_UNIT));
+		const bool repairing = reservedPad ? padStatus >= 1 : false;
+		const bool forceDisableSmooth = repairing || (aircraftState == AIRCRAFT_LANDING || aircraftState == AIRCRAFT_LANDED || (onTransportMission));
+		return !forceDisableSmooth;
+	}
+	else
+		return false;
+}
+
 void AAirMoveType::ReservePad(CAirBaseHandler::LandingPad* lp) {
 	oldGoalPos = goalPos;
 	AMoveType::ReservePad(lp);
