@@ -19,6 +19,7 @@
 #include "Map/MapDamage.h"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GL/glExtra.h"
+#include "Rendering/GL/VertexArray.h"
 #include "Rendering/UnitModels/3DModel.h"
 #include "Rendering/UnitModels/UnitDrawer.h"
 #include "Lua/LuaRules.h"
@@ -244,10 +245,9 @@ inline bool CBuilderCAI::OutOfImmobileRange(const Command& cmd) const
 	}
 	else {
 		// features don't move, but maybe the unit was transported?
-		const CFeatureSet& fset = featureHandler->GetActiveFeatures();
-		CFeatureSet::const_iterator it = fset.find(id - uh->MaxUnits());
-		if (it != fset.end()) {
-			obj = *it;
+		CFeature* feature = featureHandler->GetFeature(id - uh->MaxUnits());
+		if (feature) {
+			obj = feature;
 		}
 	}
 	if (obj == NULL) {
@@ -370,7 +370,7 @@ void CBuilderCAI::SlowUpdate()
 		return;
 	}
 
-	CBuilder* fac = (CBuilder*)owner;
+	CBuilder* builder = (CBuilder*) owner;
 	Command& c = commandQue.front();
 
 	if (OutOfImmobileRange(c)) {
@@ -382,16 +382,17 @@ void CBuilderCAI::SlowUpdate()
 	if (!owner->beingBuilt && boi != buildOptions.end()) {
 		const UnitDef* ud = unitDefHandler->GetUnitDefByName(boi->second);
 		const float radius = GetUnitDefRadius(ud, c.id);
+
 		if (inCommand) {
 			if (building) {
-				if (f3SqDist(build.pos, fac->pos) > Square(fac->buildDistance + radius - 8.0f)) {
-					owner->moveType->StartMoving(build.pos, fac->buildDistance * 0.5f + radius);
+				if (f3SqDist(build.pos, builder->pos) > Square(builder->buildDistance + radius - 8.0f)) {
+					owner->moveType->StartMoving(build.pos, builder->buildDistance * 0.5f + radius);
 				} else {
 					StopMove();
 					// needed since above startmoving cancels this
-					owner->moveType->KeepPointingTo(build.pos, (fac->buildDistance + radius) * 0.6f, false);
+					owner->moveType->KeepPointingTo(build.pos, (builder->buildDistance + radius) * 0.6f, false);
 				}
-				if (!fac->curBuild && !fac->terraforming) {
+				if (!builder->curBuild && !builder->terraforming) {
 					building = false;
 					StopMove();				//cancel the effect of KeepPointingTo
 					FinishCommand();
@@ -401,7 +402,7 @@ void CBuilderCAI::SlowUpdate()
 				else if (uh->unitsByDefs[owner->team][build.def->id].size() > build.def->maxThisUnit) {
 					// unit restricted
 					building = false;
-					fac->StopBuild();
+					builder->StopBuild();
 					CancelRestrictedUnit(boi->second);
 				}
 			} else {
@@ -417,28 +418,46 @@ void CBuilderCAI::SlowUpdate()
 					StopMove();
 				} else {
 					build.pos = helper->Pos2BuildPos(build);
-					const float sqdist = f3SqDist(build.pos, fac->pos);
+					const float sqdist = f3SqDist(build.pos, builder->pos);
 
-					if ((sqdist < Square(fac->buildDistance * 0.6f + radius)) ||
-						(!owner->unitDef->canmove && (sqdist <= Square(fac->buildDistance + radius - 8.0f)))) {
+					if ((sqdist < Square(builder->buildDistance * 0.6f + radius)) ||
+						(!owner->unitDef->canmove && (sqdist <= Square(builder->buildDistance + radius - 8.0f)))) {
 						StopMove();
 
 						if (luaRules && !luaRules->AllowUnitCreation(build.def, owner, &build.pos)) {
 							FinishCommand();
 						}
 						else if (uh->MaxUnitsPerTeam() > (int) teamHandler->Team(owner->team)->units.size()) {
-							// max unitlimit reached
+							// unit-limit not yet reached
+							CFeature* f = NULL;
 							buildRetries++;
-							owner->moveType->KeepPointingTo(build.pos, fac->buildDistance * 0.7f + radius, false);
+							owner->moveType->KeepPointingTo(build.pos, builder->buildDistance * 0.7f + radius, false);
 
-							if (fac->StartBuild(build) || (buildRetries > 20)) {
+							if (builder->StartBuild(build, f) || (buildRetries > 20)) {
 								building = true;
-							} else {
+							}
+							else if (f) {
+								inCommand = false;
+								ReclaimFeature(f);
+							}
+							else {
 								if ((owner->team == gu->myTeam) && !(buildRetries & 7)) {
-									logOutput.Print("%s: Build pos blocked", owner->unitDef->humanName.c_str());
+									logOutput.Print(
+										"%s: build-position <%.2f, %.2f, %.2f> blocked after %d attempts",
+										owner->unitDef->humanName.c_str(),
+										build.pos.x, build.pos.y, build.pos.z,
+										buildRetries
+									);
 									logOutput.SetLastMsgPos(owner->pos);
 								}
-								helper->BuggerOff(build.pos, radius, false);
+
+								const float fpSqRadius = (ud->xsize * ud->xsize + ud->zsize * ud->zsize);
+								const float fpRadius = (math::sqrt(fpSqRadius) * 0.5f) * SQUARE_SIZE;
+
+								// tell everything within the radius of the soon-to-be buildee
+								// to get out of the way; using the model radius is not correct
+								// because this can be shorter than half the footprint diagonal
+								helper->BuggerOff(build.pos, std::max(radius, fpRadius), false);
 								NonMoving();
 							}
 						}
@@ -449,7 +468,7 @@ void CBuilderCAI::SlowUpdate()
 								FinishCommand();
 							}
 						}
-						SetGoal(build.pos, owner->pos, fac->buildDistance * 0.4f + radius);
+						SetGoal(build.pos, owner->pos, builder->buildDistance * 0.4f + radius);
 					}
 				}
 			}
@@ -468,19 +487,7 @@ void CBuilderCAI::SlowUpdate()
 			uh->TestUnitBuildSquare(bi, f, owner->allyteam);
 
 			if (f) {
-				if (!owner->unitDef->canReclaim || !f->def->reclaimable) {
-					// FIXME user shouldn't be able to queue buildings on top of features
-					// in the first place (in this case).
-					StopMove();
-					FinishCommand();
-				} else {
-					Command c2;
-					c2.id=CMD_RECLAIM;
-					c2.options=0;
-					c2.params.push_back(f->id + uh->MaxUnits());
-					commandQue.push_front(c2);
-					SlowUpdate(); //this assumes that the reclaim command can never return directly without having reclaimed the target
-				}
+				ReclaimFeature(f);
 			} else {
 				inCommand=true;
 				SlowUpdate();
@@ -507,6 +514,25 @@ void CBuilderCAI::SlowUpdate()
 }
 
 
+/// add a command to reclaim a feature that is blocking our buildsite
+void CBuilderCAI::ReclaimFeature(CFeature* f)
+{
+	if (!owner->unitDef->canReclaim || !f->def->reclaimable) {
+		// FIXME user shouldn't be able to queue buildings on top of features
+		// in the first place (in this case).
+		StopMove();
+		FinishCommand();
+	} else {
+		Command c2;
+		c2.id=CMD_RECLAIM;
+		c2.options=0;
+		c2.params.push_back(f->id + uh->MaxUnits());
+		commandQue.push_front(c2);
+		SlowUpdate(); //this assumes that the reclaim command can never return directly without having reclaimed the target
+	}
+}
+
+
 void CBuilderCAI::FinishCommand(void)
 {
 	if (commandQue.front().timeOut == INT_MAX) {
@@ -518,9 +544,9 @@ void CBuilderCAI::FinishCommand(void)
 
 void CBuilderCAI::ExecuteStop(Command& c)
 {
-	CBuilder* fac=(CBuilder*)owner;
-	building=false;
-	fac->StopBuild();
+	CBuilder* builder = (CBuilder*) owner;
+	building = false;
+	builder->StopBuild();
 	CMobileCAI::ExecuteStop(c);
 }
 
@@ -792,10 +818,8 @@ void CBuilderCAI::ExecuteReclaim(Command& c)
 		}
 		const unsigned int id = signedId;
 		if (id >= uh->MaxUnits()) { // reclaim feature
-			const CFeatureSet& fset = featureHandler->GetActiveFeatures();
-			CFeatureSet::const_iterator it = fset.find(id - uh->MaxUnits());
-			if (it != fset.end()) {
-				CFeature* feature = *it;
+		CFeature* feature = featureHandler->GetFeature(id - uh->MaxUnits());
+			if (feature) {
 				if(((c.options & INTERNAL_ORDER) && !(c.options & CONTROL_KEY) && IsFeatureBeingResurrected(feature->id, owner)) ||
 					!ReclaimObject(feature)) {
 					StopMove();
@@ -891,9 +915,8 @@ void CBuilderCAI::ExecuteResurrect(Command& c)
 	if (c.params.size()==1) {
 		unsigned int id = (unsigned int) c.params[0];
 		if (id >= uh->MaxUnits()) { // resurrect feature
-			CFeatureSet::const_iterator it = featureHandler->GetActiveFeatures().find(id - uh->MaxUnits());
-			if (it != featureHandler->GetActiveFeatures().end() && (*it)->createdFromUnit != "") {
-				CFeature* feature = *it;
+		CFeature* feature = featureHandler->GetFeature(id - uh->MaxUnits());
+			if (feature && feature->createdFromUnit != "") {
 				if(((c.options & INTERNAL_ORDER) && !(c.options & CONTROL_KEY) && IsFeatureBeingReclaimed(feature->id, owner)) ||
 					!ResurrectObject(feature)) {
 					StopMove();
@@ -1010,7 +1033,7 @@ void CBuilderCAI::ExecuteFight(Command& c)
 	}
 	float3 curPosOnLine = ClosestPointOnLine(commandPos1, commandPos2, owner->pos);
 	if ((owner->unitDef->canRepair || owner->unitDef->canAssist) && // Priority 1: Repair
-	    FindRepairTargetAndRepair(curPosOnLine, 300*owner->moveState+fac->buildDistance-8, c.options, true, false)){
+	    FindRepairTargetAndRepair(curPosOnLine, 300*owner->moveState+fac->buildDistance-8, c.options, true, (c.options & META_KEY))){
 		tempOrder = true;
 		inCommand = false;
 		if (lastPC1 != gs->frameNum) {  //avoid infinite loops
@@ -1030,7 +1053,7 @@ void CBuilderCAI::ExecuteFight(Command& c)
 		return;
 	}
 	if (owner->unitDef->canReclaim && // Priority 3: Reclaim / reclaim non resurrectable (optional) / reclaim enemy units (optional)
-	    FindReclaimTargetAndReclaim(curPosOnLine, 300, c.options, false, false, (c.options & ALT_KEY), (c.options & META_KEY), (c.options & ALT_KEY))) {
+	    FindReclaimTargetAndReclaim(curPosOnLine, 300, c.options, false, true, (c.options & ALT_KEY), (c.options & META_KEY), (c.options & ALT_KEY))) {
 		tempOrder = true;
 		inCommand = false;
 		if (lastPC3 != gs->frameNum) {  //avoid infinite loops
@@ -1317,7 +1340,7 @@ bool CBuilderCAI::FindReclaimTargetAndReclaim(const float3& pos,
 		const std::vector<CFeature*> features = qf->GetFeaturesExact(pos, radius);
 		for (std::vector<CFeature*>::const_iterator fi = features.begin(); fi != features.end(); ++fi) {
 			const CFeature* f = *fi;
-			if (f->def->reclaimable && f->def->autoreclaim && (!recNonRez || !(f->def->destructable && f->createdFromUnit != "")) &&
+			if (f->def->reclaimable && ((options & CONTROL_KEY) || f->def->autoreclaim) && (!recNonRez || !(f->def->destructable && f->createdFromUnit != "")) &&
 				(recAnyTeam || (f->allyteam != owner->allyteam))) {
 				float dist = f3SqLen(f->pos - owner->pos);
 				if ((dist < bestDist) &&
@@ -1480,8 +1503,12 @@ bool CBuilderCAI::FindRepairTargetAndRepair(const float3& pos, float radius,
 		CUnit* unit = *ui;
 		if (teamHandler->Ally(owner->allyteam, unit->allyteam)) {
 			if (!haveEnemy && (unit->health < unit->maxHealth)) {
-				// dont help factories produce units unless set on roam
-				if (unit->beingBuilt && unit->mobility && (owner->moveState < 2)) {
+				// don't help allies build unless set on roam
+				if (unit->beingBuilt && owner->team != unit->team && (owner->moveState != 2)) {
+					continue;
+				}                
+				// don't help factories produce units when set on hold pos                
+				if (unit->beingBuilt && unit->mobility && (owner->moveState == 0)) {
 					continue;
 				}
 				// don't repair stuff that can't be repaired
@@ -1661,8 +1688,9 @@ void CBuilderCAI::DrawCommands(void)
 				const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
 				lineDrawer.DrawLineAndIcon(ci->id, endPos, cmdColors.restore);
 				lineDrawer.Break(endPos, cmdColors.restore);
+				glColor4fv(cmdColors.restore);
 				glSurfaceCircle(endPos, ci->params[3], 20);
-				lineDrawer.RestartSameColor();
+				lineDrawer.RestartWithColor(cmdColors.restore);
 				break;
 			}
 			case CMD_ATTACK:
@@ -1688,8 +1716,9 @@ void CBuilderCAI::DrawCommands(void)
 					const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
 					lineDrawer.DrawLineAndIcon(ci->id, endPos, color);
 					lineDrawer.Break(endPos, color);
+					glColor4fv(color);
 					glSurfaceCircle(endPos, ci->params[3], 20);
-					lineDrawer.RestartSameColor();
+					lineDrawer.RestartWithColor(color);
 				} else {
 					const int signedId = (int)ci->params[0];
 					if (signedId < 0) {
@@ -1703,10 +1732,9 @@ void CBuilderCAI::DrawCommands(void)
 
 						GML_RECMUTEX_LOCK(feat); // DrawCommands
 
-						const CFeatureSet& fset = featureHandler->GetActiveFeatures();
-						CFeatureSet::const_iterator it = fset.find(id - uh->MaxUnits());
-						if (it != fset.end()) {
-							const float3 endPos = (*it)->midPos;
+						CFeature* feature = featureHandler->GetFeature(id - uh->MaxUnits());
+						if (feature) {
+							const float3 endPos = feature->midPos;
 							lineDrawer.DrawLineAndIcon(ci->id, endPos, color);
 						}
 					} else {
@@ -1728,8 +1756,9 @@ void CBuilderCAI::DrawCommands(void)
 					const float3 endPos(ci->params[0], ci->params[1], ci->params[2]);
 					lineDrawer.DrawLineAndIcon(ci->id, endPos, color);
 					lineDrawer.Break(endPos, color);
+					glColor4fv(color);
 					glSurfaceCircle(endPos, ci->params[3], 20);
-					lineDrawer.RestartSameColor();
+					lineDrawer.RestartWithColor(color);
 				} else {
 					if (ci->params.size() >= 1) {
 						const unsigned int uid = (unsigned int) ci->params[0];
@@ -1771,49 +1800,138 @@ void CBuilderCAI::DrawCommands(void)
 }
 
 
+
+// XXX move away from this class
 void CBuilderCAI::DrawQuedBuildingSquares(void)
 {
 	CCommandQueue::const_iterator ci;
+	// worst case - 2 squares per building (when underwater) - 8 vertices * 3 floats
+
+	int buildCommands = 0;
+	int underwaterCommands = 0;
+	for (ci = commandQue.begin(); ci != commandQue.end(); ++ci) {
+		if (buildOptions.find(ci->id) != buildOptions.end()) {
+			++buildCommands;
+			BuildInfo bi(*ci);
+			bi.pos = helper->Pos2BuildPos(bi);
+			if (bi.pos.y < 0.f)
+				++underwaterCommands;
+		}
+	}
+	std::vector<GLfloat> vertices_quads(buildCommands * 12);
+	std::vector<GLfloat> vertices_quads_uw(buildCommands * 12); // underwater
+	// 4 vertical lines
+	std::vector<GLfloat> vertices_lines(underwaterCommands * 24);
+	// colors for lines
+	std::vector<GLfloat> colors_lines(underwaterCommands * 48);
+
+	int quadcounter = 0;
+	int uwqcounter = 0;
+	int linecounter = 0;
 	for (ci = commandQue.begin(); ci != commandQue.end(); ++ci) {
 		if (buildOptions.find(ci->id) != buildOptions.end()) {
 			BuildInfo bi(*ci);
 			bi.pos = helper->Pos2BuildPos(bi);
 			const float xsize = bi.GetXSize()*4;
 			const float zsize = bi.GetZSize()*4;
-			glBegin(GL_LINE_LOOP);
-			glVertexf3(bi.pos + float3(+xsize, 1, +zsize));
-			glVertexf3(bi.pos + float3(-xsize, 1, +zsize));
-			glVertexf3(bi.pos + float3(-xsize, 1, -zsize));
-			glVertexf3(bi.pos + float3(+xsize, 1, -zsize));
-			glEnd();
+
+			const float h = bi.pos.y;
+			const float x1 = bi.pos.x - xsize;
+			const float z1 = bi.pos.z - zsize;
+			const float x2 = bi.pos.x + xsize;
+			const float z2 = bi.pos.z + zsize;
+
+			vertices_quads[quadcounter + 0] = x1;
+			vertices_quads[quadcounter + 1] = h + 1;
+			vertices_quads[quadcounter + 2] = z1;
+			vertices_quads[quadcounter + 3] = x1;
+			vertices_quads[quadcounter + 4] = h + 1;
+			vertices_quads[quadcounter + 5] = z2;
+			vertices_quads[quadcounter + 6] = x2;
+			vertices_quads[quadcounter + 7] = h + 1;
+			vertices_quads[quadcounter + 8] = z2;
+			vertices_quads[quadcounter + 9] = x2;
+			vertices_quads[quadcounter +10] = h + 1;
+			vertices_quads[quadcounter +11] = z1;
+
+			quadcounter += 12;
+
 			if (bi.pos.y < 0.0f) {
-				const float s[4] = { 0.0f, 0.0f, 1.0f, 0.5f }; // start color
-				const float e[4] = { 0.0f, 0.5f, 1.0f, 1.0f }; // end color
+				const float col[8] = { 0.0f, 0.0f, 1.0f, 0.5f, // start color
+						       0.0f, 0.5f, 1.0f, 1.0f }; // end color
 
-				const float h = bi.pos.y;
-				const float x1 = bi.pos.x - xsize;
-				const float z1 = bi.pos.z - zsize;
-				const float x2 = bi.pos.x + xsize;
-				const float z2 = bi.pos.z + zsize;
+				vertices_quads_uw[uwqcounter + 0] = x1;
+				vertices_quads_uw[uwqcounter + 1] = 0.f;
+				vertices_quads_uw[uwqcounter + 2] = z1;
+				vertices_quads_uw[uwqcounter + 3] = x1;
+				vertices_quads_uw[uwqcounter + 4] = 0.f;
+				vertices_quads_uw[uwqcounter + 5] = z2;
+				vertices_quads_uw[uwqcounter + 6] = x2;
+				vertices_quads_uw[uwqcounter + 7] = 0.f;
+				vertices_quads_uw[uwqcounter + 8] = z2;
+				vertices_quads_uw[uwqcounter + 9] = x2;
+				vertices_quads_uw[uwqcounter +10] = 0.f;
+				vertices_quads_uw[uwqcounter +11] = z1;
 
-				glPushAttrib(GL_CURRENT_BIT);
-				glBegin(GL_LINES);
-				glColor4fv(s); glVertex3f(x1, h, z1); glColor4fv(e); glVertex3f(x1, 0.0f, z1);
-				glColor4fv(s); glVertex3f(x2, h, z1); glColor4fv(e); glVertex3f(x2, 0.0f, z1);
-				glColor4fv(s); glVertex3f(x1, h, z2); glColor4fv(e); glVertex3f(x1, 0.0f, z2);
-				glColor4fv(s); glVertex3f(x2, h, z2); glColor4fv(e); glVertex3f(x2, 0.0f, z2);
-				glEnd();
-				// using the last end color
-				glBegin(GL_LINE_LOOP);
-				glVertex3f(x1, 0.0f, z1);
-				glVertex3f(x1, 0.0f, z2);
-				glVertex3f(x2, 0.0f, z2);
-				glVertex3f(x2, 0.0f, z1);
-				glEnd();
-				glPopAttrib();
+				uwqcounter += 12;
+
+				for (int i = 0; i<4; ++i) {
+					std::copy(col, col + 8, colors_lines.begin() + linecounter * 2 + i * 8);
+				}
+
+				vertices_lines[linecounter + 0] = x1;
+				vertices_lines[linecounter + 1] = h;
+				vertices_lines[linecounter + 2] = z1;
+				vertices_lines[linecounter + 3] = x1;
+				vertices_lines[linecounter + 4] = 0;
+				vertices_lines[linecounter + 5] = z1;
+
+				vertices_lines[linecounter + 6] = x2;
+				vertices_lines[linecounter + 7] = h;
+				vertices_lines[linecounter + 8] = z1;
+				vertices_lines[linecounter + 9] = x2;
+				vertices_lines[linecounter +10] = 0;
+				vertices_lines[linecounter +11] = z1;
+
+				vertices_lines[linecounter +12] = x2;
+				vertices_lines[linecounter +13] = h;
+				vertices_lines[linecounter +14] = z2;
+				vertices_lines[linecounter +15] = x2;
+				vertices_lines[linecounter +16] = 0;
+				vertices_lines[linecounter +17] = z2;
+
+				vertices_lines[linecounter +18] = x1;
+				vertices_lines[linecounter +19] = h;
+				vertices_lines[linecounter +20] = z2;
+				vertices_lines[linecounter +21] = x1;
+				vertices_lines[linecounter +22] = 0;
+				vertices_lines[linecounter +23] = z2;
+
+				linecounter += 24;
 			}
 		}
 	}
+	if (quadcounter) {
+		glEnableClientState(GL_VERTEX_ARRAY);
+		glPolygonMode(GL_FRONT_AND_BACK, GL_LINES);
+		glVertexPointer(3, GL_FLOAT, 0, &vertices_quads[0]);
+		glDrawArrays(GL_QUADS, 0, quadcounter/3);
+
+		if (linecounter) {
+			glPushAttrib(GL_CURRENT_BIT);
+			glColor4f(0.0f, 0.5f, 1.0f, 1.0f); // same as end color of lines
+			glVertexPointer(3, GL_FLOAT, 0, &vertices_quads_uw[0]);
+			glDrawArrays(GL_QUADS, 0, uwqcounter/3);
+			glPopAttrib();
+
+			glEnableClientState(GL_COLOR_ARRAY);
+			glColorPointer(4, GL_FLOAT, 0, &colors_lines[0]);
+			glVertexPointer(3, GL_FLOAT, 0, &vertices_lines[0]);
+			glDrawArrays(GL_LINES, 0, linecounter/3);
+			glDisableClientState(GL_COLOR_ARRAY);
+		}
+		glDisableClientState(GL_VERTEX_ARRAY);
+	}
 }
 
 
