@@ -17,6 +17,7 @@
 #include "Rendering/UnitModels/3DOParser.h"
 #include "Sim/Features/Feature.h"
 #include "Sim/Features/FeatureHandler.h"
+#include "Sim/Misc/GroundBlockingObjectMap.h"
 #include "Sim/Misc/ModInfo.h"
 #include "Sim/Misc/TeamHandler.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
@@ -115,9 +116,10 @@ void CBuilder::PostLoad()
 void CBuilder::UnitInit(const UnitDef* def, int team, const float3& position)
 {
 	range3D = def->buildRange3D;
-	buildDistance  = def->buildDistance;
+	buildDistance = def->buildDistance;
 
 	const float scale = (1.0f / 32.0f);
+
 	buildSpeed     = scale * def->buildSpeed;
 	repairSpeed    = scale * def->repairSpeed;
 	reclaimSpeed   = scale * def->reclaimSpeed;
@@ -536,28 +538,46 @@ void CBuilder::StopBuild(bool callScript)
 }
 
 
-bool CBuilder::StartBuild(BuildInfo& buildInfo)
+bool CBuilder::StartBuild(BuildInfo& buildInfo, CFeature*& feature)
 {
 	StopBuild(false);
-//	logOutput.Print("start build");
 
-	buildInfo.pos=helper->Pos2BuildPos(buildInfo);
+	buildInfo.pos = helper->Pos2BuildPos(buildInfo);
 
-	CFeature* feature;
 	// Pass -1 as allyteam to behave like we have maphack.
 	// This is needed to prevent building on top of cloaked stuff.
-	int canBuild=uh->TestUnitBuildSquare(buildInfo, feature, -1);
-	if(canBuild<2){
-		CUnit* u=helper->GetClosestFriendlyUnit(buildInfo.pos,5,allyteam);
-		if(u && u->unitDef==buildInfo.def && unitDef->canAssist){
-			curBuild=u;
+	const int canBuild = uh->TestUnitBuildSquare(buildInfo, feature, -1);
+
+	if (canBuild < 2) {
+		// the ground is blocked at the position we want
+		// to build at; check if the blocking object is
+		// of the same type as our buildee (which means
+		// another builder has already started it)
+		// note: even if construction has already started,
+		// the buildee is *not* guaranteed to be the unit
+		// closest to us
+		CSolidObject* o = groundBlockingObjectMap->GroundBlocked(buildInfo.pos);
+		CUnit* u = NULL;
+
+		if (o != NULL) {
+			u = dynamic_cast<CUnit*>(o);
+		} else {
+			// <pos> might map to a non-blocking portion
+			// of the buildee's yardmap, fallback check
+			u = helper->GetClosestFriendlyUnit(buildInfo.pos, buildDistance, allyteam);
+		}
+
+		if (u != NULL && u->unitDef == buildInfo.def && unitDef->canAssist) {
+			curBuild = u;
 			AddDeathDependence(u);
 			SetBuildStanceToward(buildInfo.pos);
 			return true;
 		}
+
 		return false;
 	}
-	if(feature)
+
+	if (feature)
 		return false;
 
 	const UnitDef* unitDef = buildInfo.def;
@@ -583,11 +603,11 @@ bool CBuilder::StartBuild(BuildInfo& buildInfo)
 		tz2 = min(gs->mapy,tz1+b->unitDef->zsize);
 
 		b->terraformLeft = CalculateBuildTerraformCost(buildInfo);
-		b->groundLevelled=false;
+		b->groundLevelled= false;
 
-		terraforming=true;
-		terraformType=Terraform_Building;
-		terraformRadius=(tx2-tx1)*SQUARE_SIZE;
+		terraforming    = true;
+		terraformType   = Terraform_Building;
+		terraformRadius = (tx2-tx1)*SQUARE_SIZE;
 		terraformCenter = b->pos;
 	}
 
@@ -598,26 +618,16 @@ bool CBuilder::StartBuild(BuildInfo& buildInfo)
 	b->lineage = this->lineage;
 	AddDeathDependence(b);
 	curBuild=b;
-	if (mapDamage->disabled && !(curBuild->floatOnWater)) {
-		/* The ground isn't going to be terraformed.
-		 * When the building is completed, it'll 'pop'
-		 * into the correct height for the (un-flattened)
-		 * terrain it's on.
-		 *
-		 * To prevent this visual artifact, put the building
-		 * at the 'right' height to begin with.
-		 *
-		 * Duplicated from CMoveType::SlowUpdate(), which
-		 * is why we use the regular code for floating things.
-		 */
-		curBuild->pos.y = groundheight;
-		curBuild->midPos.y = groundheight + curBuild->relMidPos.y;
-	}
-	else {
-		float d=buildInfo.pos.y-curBuild->pos.y;
-		curBuild->pos.y+=d;
-		curBuild->midPos.y+=d;
-	}
+
+	/* The ground isn't going to be terraformed.
+	 * When the building is completed, it'll 'pop'
+	 * into the correct height for the (un-flattened)
+	 * terrain it's on.
+	 *
+	 * To prevent this visual artifact, put the building
+	 * at the 'right' height to begin with.
+	 */
+	curBuild->moveType->SlowUpdate();
 
 	return true;
 }
