@@ -24,11 +24,27 @@
 	#define NO_RETURN
 #endif
 
+// legacy support define
 #define handleerror(o, m, c, f) ErrorMessageBox(m, c, f)
-void ErrorMessageBox(const char *msg, const char *caption, unsigned int flags) NO_RETURN;
-inline void ErrorMessageBox(const std::string& msg, const std::string& caption, unsigned int flags)
+
+/**
+ * Will pop up an error message window and exit (C version).
+ * @param  msg     the main text, describing the error
+ * @param  caption will appear in the title bar of the error window
+ * @param  flags   one of:
+ *                 - MB_OK   : Error
+ *                 - MB_EXCL : Warning
+ *                 - MB_INFO : Info
+ */
+void ErrorMessageBox(const char* msg, const char* caption, unsigned int flags) NO_RETURN;
+
+/**
+ * Will pop up an error message window and exit (C++ version).
+ * @see ErrorMessageBox(const char* msg, const char* caption, unsigned int flags)
+ */
+inline void ErrorMessageBox(const std::string& msg, const std::string& caption, unsigned int flags = MBF_OK)
 {
 	ErrorMessageBox(msg.c_str(), caption.c_str(), flags);
 };
 
-#endif
+#endif // ERRORHANDLER_H
