@@ -255,7 +255,7 @@ static	void	doCleanupLogOnFirstRun()
 
 static	const char	*sourceFileStripper(const char *sourceFile)
 {
-	char	*ptr = strrchr(sourceFile, '\\');
+	const char	*ptr = strrchr(sourceFile, '\\');
 	if (ptr) return ptr + 1;
 	ptr = strrchr(sourceFile, '/');
 	if (ptr) return ptr + 1;
