@@ -2,6 +2,7 @@
 #define LINEEDIT_H
 
 #include <string>
+#include <boost/signal.hpp>
 
 #include "GuiElement.h"
 
@@ -20,12 +21,16 @@ public:
 	void SetContent(const std::string& line, bool moveCursor = true);
 	
 	void SetFocus(bool focus);
+	void SetCrypt(bool focus);
+
+	boost::signal<void (void)> DefaultAction;
 
 private:
 	virtual void DrawSelf();
 	virtual bool HandleEventSelf(const SDL_Event& ev);
 
 	bool hasFocus;
+	bool crypt;
 	std::string content;
 	unsigned cursorPos;
 };
